@@ -1,5 +1,5 @@
 # scp.py
-# Copyright (C) 2008 James Bardin <jbardin@bu.edu>
+# Copyright (C) 2008 James Bardin <j.bardin@gmail.com>
 
 """
 Utilities for sending files over ssh using the scp1 protocol.
@@ -11,6 +11,21 @@ from socket import timeout as SocketTimeout
 
 DEBUG = False
 
+# this is quote from the shlex module, added in py3.3
+_find_unsafe = re.compile(r'[^\w@%+=:,./~-]').search
+
+
+def _sh_quote(s):
+    """Return a shell-escaped version of the string *s*."""
+    if not s:
+        return ""
+    if _find_unsafe(s) is None:
+        return s
+
+    # use single quotes, and put single quotes into double quotes
+    # the string $'b is then quoted as '$'"'"'b'
+    return "'" + s.replace("'", "'\"'\"'") + "'"
+
 
 class SCPClient(object):
     """
@@ -27,7 +42,7 @@ class SCPClient(object):
     (matching scp behaviour), but we make no attempt at symlinked directories.
     """
     def __init__(self, transport, buff_size=16384, socket_timeout=5.0,
-                 progress=None):
+                 progress=None, sanitize=_sh_quote):
         """
         Create an scp1 client.
 
@@ -39,6 +54,8 @@ class SCPClient(object):
         @type socket_timeout: float
         @param progress: callback - called with (filename, size, sent) during
             transfers
+        @param sanitize: function - called with filename, should return
+            safe or escaped string.  Uses _sh_quote by default.
         @type progress: function(string, int, int)
         """
         self.transport = transport
@@ -50,6 +67,7 @@ class SCPClient(object):
         self._recv_dir = ''
         self._rename = False
         self._utime = None
+        self.sanitize = sanitize
         self._dirtimes = {}
 
     def put(self, files, remote_path='.',
@@ -73,7 +91,7 @@ class SCPClient(object):
         self.channel = self.transport.open_session()
         self.channel.settimeout(self.socket_timeout)
         scp_command = ('scp -t %s', 'scp -r -t %s')[recursive]
-        self.channel.exec_command(scp_command % _sh_quote(remote_path))
+        self.channel.exec_command(scp_command % self.sanitize(remote_path))
         self._recv_confirm()
 
         if not isinstance(files, (list, tuple)):
@@ -106,7 +124,7 @@ class SCPClient(object):
         """
         if not isinstance(remote_path, (list, tuple)):
             remote_path = [remote_path]
-        remote_path = [_sh_quote(r) for r in remote_path]
+        remote_path = [self.sanitize(r) for r in remote_path]
         self._recv_dir = local_path or os.getcwd()
         self._rename = len(remote_path) == 1 and not os.path.isdir(local_path)
         if len(remote_path) > 1:
@@ -142,7 +160,7 @@ class SCPClient(object):
             (mode, size, mtime, atime) = self._read_stats(name)
             if self.preserve_times:
                 self._send_time(mtime, atime)
-            file_hdl = file(name, 'rb')
+            file_hdl = open(name, 'rb')
 
             # The protocol can't handle \n in the filename.
             # Quote them as the control sequence \^J for now,
@@ -226,9 +244,10 @@ class SCPClient(object):
             msg = self.channel.recv(512)
         except SocketTimeout:
             raise SCPException('Timout waiting for scp response')
-        if msg and msg[0] == '\x00':
+        # slice off the first byte, so this compare will work in python2 and python3
+        if msg and msg[0:1] == b'\x00':
             return
-        elif msg and msg[0] == '\x01':
+        elif msg and msg[0:1] == b'\x01':
             raise SCPException(msg[1:])
         elif self.channel.recv_stderr_ready():
             msg = self.channel.recv_stderr(512)
@@ -236,21 +255,21 @@ class SCPClient(object):
         elif not msg:
             raise SCPException('No response from server')
         else:
-            raise SCPException('Invalid response from server: ' + msg)
+            raise SCPException('Invalid response from server', msg)
 
     def _recv_all(self):
         # loop over scp commands, and recive as necessary
-        command = {'C': self._recv_file,
-                   'T': self._set_time,
-                   'D': self._recv_pushd,
-                   'E': self._recv_popd}
+        command = {b'C': self._recv_file,
+                   b'T': self._set_time,
+                   b'D': self._recv_pushd,
+                   b'E': self._recv_popd}
         while not self.channel.closed:
             # wait for command as long as we're open
             self.channel.sendall('\x00')
             msg = self.channel.recv(1024)
             if not msg:  # chan closed while recving
                 break
-            code = msg[0]
+            code = msg[0:1]
             try:
                 command[code](msg[1:])
             except KeyError:
@@ -264,7 +283,7 @@ class SCPClient(object):
             mtime = int(times[0])
             atime = int(times[2]) or mtime
         except:
-            self.channel.send('\x01')
+            self.channel.send(b'\x01')
             raise SCPException('Bad time format')
         # save for later
         self._utime = (atime, mtime)
@@ -285,9 +304,9 @@ class SCPClient(object):
             raise SCPException('Bad file format')
 
         try:
-            file_hdl = file(path, 'wb')
+            file_hdl = open(path, 'wb')
         except IOError as e:
-            chan.send('\x01' + str(e))
+            chan.send(b'\x01' + str(e).encode())
             chan.close()
             raise
 
@@ -299,7 +318,7 @@ class SCPClient(object):
                 self._progress(path, size, 0)
         buff_size = self.buff_size
         pos = 0
-        chan.send('\x00')
+        chan.send(b'\x00')
         try:
             while pos < size:
                 # we have to make sure we don't read the final byte
@@ -311,7 +330,7 @@ class SCPClient(object):
                     self._progress(path, size, pos)
 
             msg = chan.recv(512)
-            if msg and msg[0] != '\x00':
+            if msg and msg[0:1] != b'\x00':
                 raise SCPException(msg[1:])
         except SocketTimeout:
             chan.close()
@@ -336,7 +355,7 @@ class SCPClient(object):
                 path = self._recv_dir
                 self._rename = False
         except:
-            self.channel.send('\x01')
+            self.channel.send(b'\x01')
             raise SCPException('Bad directory format')
         try:
             if not os.path.exists(path):
@@ -349,7 +368,7 @@ class SCPClient(object):
             self._utime = None
             self._recv_dir = path
         except (OSError, SCPException) as e:
-            self.channel.send('\x01' + str(e))
+            self.channel.send(b'\x01' + str(e).encode())
             raise
 
     def _recv_popd(self, *cmd):
@@ -366,19 +385,3 @@ class SCPClient(object):
 class SCPException(Exception):
     """SCP exception class"""
     pass
-
-
-# this is quote from the shlex module, added in py3.3
-_find_unsafe = re.compile(r'[^\w@%+=:,./~-]').search
-
-
-def _sh_quote(s):
-    """Return a shell-escaped version of the string *s*."""
-    if not s:
-        return ""
-    if _find_unsafe(s) is None:
-        return s
-
-    # use single quotes, and put single quotes into double quotes
-    # the string $'b is then quoted as '$'"'"'b'
-    return "'" + s.replace("'", "'\"'\"'") + "'"
